@@ -11,11 +11,10 @@ import rich.logging
 from rich import traceback
 from rich import print
 
-import mlf_core
 from mlf_core.bump_version.bump_version import VersionBumper
 from mlf_core.config.config import ConfigCommand
 from mlf_core.create.create import choose_domain
-from mlf_core.custom_cli.click import HelpErrorHandling, print_project_version, CustomHelpSubcommand, CustomArg
+from mlf_core.custom_cli.click import HelpErrorHandling, print_project_version, print_mlfcore_version, CustomHelpSubcommand, CustomArg
 from mlf_core.info.info import TemplateInfo
 from mlf_core.lint.lint import lint_project
 from mlf_core.list.list import TemplateLister
@@ -48,7 +47,7 @@ def main():
 
 
 @click.group(cls=HelpErrorHandling)
-@click.version_option(mlf_core.__version__, message=click.style(f'mlf-core Version: {mlf_core.__version__}', fg='blue'))
+@click.option('--version', is_flag=True, callback=print_mlfcore_version, expose_value=False, is_eager=True, help='Print the current mlf-core version.')
 @click.option('-v', '--verbose', is_flag=True, default=False, help='Enable verbose output (print debug statements).')
 @click.option("-l", "--log-file", help="Save a verbose log to a file.")
 @click.pass_context
@@ -78,9 +77,10 @@ def mlf_core_cli(ctx, verbose, log_file):
 
 
 @mlf_core_cli.command(short_help='Create a new project using one of our templates.', cls=CustomHelpSubcommand)
+@click.argument('path', type=click.Path(), default=Path.cwd(), helpmsg='Path where the project should be created at.', cls=CustomArg)  # type: ignore
 @click.option('--domain', type=click.Choice(['cli', 'lib', 'gui', 'web', 'pub']),
               help='The projects domain with currently cli, lib, gui, web and pub supported.')
-def create(domain: str) -> None:
+def create(path: Path, domain: str) -> None:
     """
     Create a new project using one of our templates.
 
@@ -90,7 +90,7 @@ def create(domain: str) -> None:
     Next, you will be asked whether you want to use mlf-core's Github support create a repository, push your template and enable a few settings.
     After the project has been created it will be linted and you will be notified of any TODOs.
     """
-    choose_domain(domain, None)
+    choose_domain(path, domain, None)
 
 
 @mlf_core_cli.command(short_help='Lint your existing mlf-core project.', cls=CustomHelpSubcommand)
@@ -152,13 +152,16 @@ def sync(project_dir, set_token, pat, username, check_update) -> None:
     To ensure that you have the latest changes you can invoke sync, which submits a pull request to your Github repository (if existing).
     If no repository exists the TEMPLATE branch will be updated and you can merge manually.
     """
-    project_dir_path = Path(f'{Path.cwd()}/{project_dir}') if not str(project_dir).startswith(str(Path.cwd())) else Path(project_dir)
+    project_dir_path = Path(project_dir).resolve()
+    log.debug(f'Loading project information from .mlf_core.yml file located at {project_dir}')
     project_data = load_yaml_file(f'{project_dir}/.mlf_core.yml')
-
+    log.debug(f'Set project top level path to given path argument {project_dir_path}')
     # if set_token flag is set, update the sync token value and exit
     if set_token:
+        log.debug('Running sync to update sync token in repo.')
         try:
             if project_data['is_github_repo']:
+                log.debug(f'Project is a Github repo. Using {project_data["github_username"]} as username.')
                 TemplateSync.update_sync_token(project_name=project_data['project_slug'], gh_username=project_data['github_username'])
             else:
                 print('[bold red]Your current project does not seem to have a Github repository!')
@@ -170,12 +173,15 @@ def sync(project_dir, set_token, pat, username, check_update) -> None:
             sys.exit(1)
         sys.exit(0)
 
-    syncer = TemplateSync(new_template_version='', project_dir=project_dir_path, gh_username=username, token=pat, repo_owner=project_data['github_username'])
+    log.debug('Initializing syncer object.')
+    syncer = TemplateSync(new_template_version='', project_dir=project_dir_path, gh_username=username, token=pat)
     # check for template version updates
+    log.debug('Checking for major/minor or patch version changes in mlf-core templates.')
     major_change, minor_change, patch_change, project_template_version, mlf_core_template_version = syncer.has_template_version_changed(project_dir_path)
     syncer.new_template_version = mlf_core_template_version
     # check for user without actually syncing
     if check_update:
+        log.debug('Running snyc to manually check whether a new template version is available.')
         # a template update has been released by mlf-core
         if any(change for change in (major_change, minor_change, patch_change)):
             print(f'[bold blue]Your templates version received an update from {project_template_version} to {mlf_core_template_version}!\n'
@@ -189,14 +195,17 @@ def sync(project_dir, set_token, pat, username, check_update) -> None:
     syncer.major_update = major_change
     syncer.minor_update = minor_change
     syncer.patch_update = patch_change
+    log.debug('Major template update found.' if major_change else 'Minor template update found.' if minor_change else 'Patch template update found.' if
+              patch_change else 'No template update found.')
 
     # sync the project if any changes were detected
     if any(change for change in (major_change, minor_change, patch_change)):
-        # If required sync level is set -> sync
-        if syncer.check_sync_level():
+        # If required sync level is set and sync is enabled -> sync
+        if syncer.should_run_sync():
+            log.debug('Starting sync.')
             syncer.sync()
         else:
-            print('[bold red]Aborting sync due to set level constraints. '
+            print('[bold red]Aborting sync due to set level constraints or sync being disabled. '
                   'You can set the level any time in your mlf_core.cfg in the sync_level section and sync again.')
     else:
         print('[bold blue]No changes detected. Your template is up to date.')
@@ -240,7 +249,7 @@ def bump_version(ctx, new_version, project_dir, downgrade) -> None:
                     version_bumper.bump_template_version(new_version, project_dir)
                 elif mlf_core_questionary_or_dot_mlf_core(function='confirm',
                                                           question=f'Bumping from {version_bumper.CURRENT_VERSION} to {new_version} seems not reasonable.\n'
-                                                          f'Do you really want to bump the project version?',
+                                                                   f'Do you really want to bump the project version?',
                                                           default='n'):
                     print('\n')
                     version_bumper.bump_template_version(new_version, project_dir)
@@ -285,15 +294,18 @@ def config(ctx, view: bool, section: str) -> None:
 
 
 @mlf_core_cli.command(short_help='Fix artifact location path for local all mlruns.', cls=CustomHelpSubcommand)
-@click.argument('path', type=str, default='.', required=False, helpmsg='Path to the root of the mlruns folder.', cls=CustomArg)
+@click.argument('path', type=str, default=os.getcwd(), required=False, helpmsg='Path to the root of the mlruns folder.', cls=CustomArg)
 @click.pass_context
 def fix_artifact_paths(ctx, path: str) -> None:
     """
+    Ensures that the paths of all locally saved MLflow artifacts are fixed to display them on the current machine.
     """
     for meta_yaml in Path(f'{path}/mlruns').rglob('meta.yaml'):
-        if 'file' not in meta_yaml.absolute():
-            print(f'[bold yellow] Skipping path fixing for: {meta_yaml.absolute()}. Run was not saved locally.')
-        print(f'[bold blue] Fixing path for: {meta_yaml.absolute()}')
+        with open(meta_yaml.absolute()) as meta_yaml_file:
+            content = meta_yaml_file.readlines()
+            if 'file://' not in content[0]:
+                print(f'[bold yellow]Skipping path fixing for: {meta_yaml.absolute()}. Run was not saved locally.')
+        print(f'[bold blue]Fixing path for: {meta_yaml.absolute()}')
         with open(meta_yaml.absolute()) as meta_yaml_file:
             content = meta_yaml_file.readlines()
             if 'artifact_location' in content[0]:
@@ -314,5 +326,6 @@ def upgrade() -> None:
     UpgradeCommand.check_upgrade_mlf_core()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
+    traceback.install()
     sys.exit(main())  # pragma: no cover
