@@ -1,5 +1,6 @@
 import os
 import sys
+import logging
 import appdirs
 from pathlib import Path
 from cryptography.fernet import Fernet
@@ -14,6 +15,8 @@ from mlf_core.common.levensthein_dist import most_similar_command
 from mlf_core.custom_cli.questionary import mlf_core_questionary_or_dot_mlf_core
 from mlf_core.common.load_yaml import load_yaml_file
 
+log = logging.getLogger(__name__)
+
 
 class ConfigCommand:
     """
@@ -28,28 +31,37 @@ class ConfigCommand:
         """
         Set general settings and PAT.
         """
-        ConfigCommand.check_ct_config_dir_exists()
+        if not ConfigCommand.check_mlf_core_config_dir_exists():
+            ConfigCommand.create_config_dirs()
         ConfigCommand.config_general_settings()
         ConfigCommand.config_pat()
 
     @staticmethod
     def config_general_settings() -> None:
         """
-        Set full_name and email for reuse in any project created further on.
+        Set full_name, email and github username for reuse in any project created further on. Defaults for prompts
+        are set to the previous configured values (if any).
         """
-        ConfigCommand.check_ct_config_dir_exists()
+        already_configured = False
+        settings = {}
+        config_path = Path(ConfigCommand.CONF_FILE_PATH)
+        if config_path.exists() and 'GITHUB_ACTIONS' not in os.environ:
+            already_configured = True
+            settings = load_yaml_file(ConfigCommand.CONF_FILE_PATH)
+
         full_name = mlf_core_questionary_or_dot_mlf_core(function='text',
                                                          question='Full name',
-                                                         default='Homer Simpson')
+                                                         default='Homer Simpson' if not already_configured else settings['full_name'])
         email = mlf_core_questionary_or_dot_mlf_core(function='text',
                                                      question='Personal or work email',
-                                                     default='homer.simpson@example.com')
+                                                     default='homer.simpson@example.com' if not already_configured else settings['email'])
         github_username = mlf_core_questionary_or_dot_mlf_core(function='text',
                                                                question='Github username',
-                                                               default='HomerGithub').lower()
+                                                               default='HomerGithub' if not already_configured else settings['github_username']).lower()
 
         # if the configs exist, just update them
         if os.path.exists(ConfigCommand.CONF_FILE_PATH):
+            log.debug(f'Configuration was found at {ConfigCommand.CONF_FILE_PATH}. Updating configuration...')
             path = Path(ConfigCommand.CONF_FILE_PATH)
             yaml = YAML()
             settings = yaml.load(path)
@@ -62,6 +74,7 @@ class ConfigCommand:
 
         # the configs don´t exist -> create them
         else:
+            log.debug(f'Configuration was not found at {ConfigCommand.CONF_FILE_PATH}. Creating a new configuration file.')
             settings = {'full_name': full_name, 'email': email, 'github_username': github_username}
             yaml = YAML()
             yaml.dump(settings, Path(ConfigCommand.CONF_FILE_PATH))
@@ -71,7 +84,9 @@ class ConfigCommand:
         """
         Set the personal access token (PAT) for automatic Github repo creation.
         """
-        ConfigCommand.check_ct_config_dir_exists()
+        if not ConfigCommand.check_mlf_core_config_dir_exists():
+            ConfigCommand.create_config_dirs()
+
         try:
             path = Path(ConfigCommand.CONF_FILE_PATH)
             yaml = YAML()
@@ -88,8 +103,8 @@ class ConfigCommand:
 
         if mlf_core_questionary_or_dot_mlf_core(function='confirm',
                                                 question='Do you want to configure your GitHub personal access token right now?\n'
-                                                'You can still configure it later '
-                                                'by calling    mlf-core config pat',
+                                                         'You can still configure it later '
+                                                         'by calling    mlf-core config pat',
                                                 default='Yes'):
             print('[bold blue]mlf-core requires your Github Access token to have full repository, workflow and create/update packages permissions!')
             access_token = mlf_core_questionary_or_dot_mlf_core(function='password',
@@ -105,8 +120,10 @@ class ConfigCommand:
 
             # encrypt the given PAT and save the encryption key and encrypted PAT in separate files
             print('[bold blue]Generating key for encryption.')
+            log.debug('Generating personal access key.')
             key = Fernet.generate_key()
             fer = Fernet(key)
+            log.debug('Encrypting personal access token. ')
             print('[bold blue]Encrypting personal access token.')
             encrypted_pat = fer.encrypt(access_token_b)
 
@@ -118,16 +135,19 @@ class ConfigCommand:
             yaml = YAML()
             settings = yaml.load(path)
             settings['pat'] = encrypted_pat
+            log.debug(f'Dumping configuration to {ConfigCommand.CONF_FILE_PATH}')
             yaml.dump(settings, Path(ConfigCommand.CONF_FILE_PATH))
 
     @staticmethod
     def view_current_config() -> None:
         """
-        Print the current users cookietemple configuration.
+        Print the current users mlf-core configuration.
         """
         # load current settings
         try:
+            log.debug(f'Fetching current cookietemple configuration at {ConfigCommand.CONF_FILE_PATH}.')
             settings = load_yaml_file(ConfigCommand.CONF_FILE_PATH)
+            log.debug('Creating configuration table')
             # create the table and print
             table = Table(title="[bold]Your current configuration", title_style="blue", header_style=Style(color="blue", bold=True), box=HEAVY_HEAD)
             table.add_column('Name', style='green')
@@ -175,12 +195,22 @@ class ConfigCommand:
             print('[bold red]Unknown handle. [green]See mlf-core --help [red]for more information on valid handles')
 
     @staticmethod
-    def check_ct_config_dir_exists() -> None:
+    def check_mlf_core_config_dir_exists() -> bool:
         """
-        Check if the config directory for mlf-core exists. If not, create it.
+        Check whether the config directory for mlf-core exists.
         """
+        log.debug(f'Checking whether a config directory already exists at {Path(ConfigCommand.CONF_FILE_PATH).parent}.')
         if not os.path.exists(Path(ConfigCommand.CONF_FILE_PATH).parent):
-            os.mkdir(Path(ConfigCommand.CONF_FILE_PATH).parent)
+            log.debug('Config directory did not exist. Creating it.')
+            return False
+        return True
+
+    @staticmethod
+    def create_config_dirs() -> None:
+        """
+        Create the config directory, if none exists (and parent directories, if necessary)
+        """
+        os.makedirs(Path(ConfigCommand.CONF_FILE_PATH).parent)
 
     @staticmethod
     def handle_switcher() -> dict:
