@@ -1,4 +1,5 @@
 import sys
+import logging
 from pathlib import Path
 from ruamel.yaml import YAML
 from rich import print
@@ -7,6 +8,8 @@ from mlf_core.lint.domains.mlflow import MlflowPytorchLint, MlflowTensorflowLint
 from mlf_core.lint.domains.package import PackagePredictionLint
 from mlf_core.lint.template_linter import TemplateLinter
 
+log = logging.getLogger(__name__)
+
 
 def lint_project(project_dir: str) -> TemplateLinter:
     """
@@ -16,6 +19,7 @@ def lint_project(project_dir: str) -> TemplateLinter:
     """
     # Detect which template the project is based on
     template_handle = get_template_handle(project_dir)
+    log.debug(f'Detected handle {template_handle}')
 
     switcher = {
         'mlflow-pytorch': MlflowPytorchLint,
@@ -34,10 +38,12 @@ def lint_project(project_dir: str) -> TemplateLinter:
     # Run the linting tests
     try:
         # Run non project specific linting
+        log.debug('Running general linting.')
         print('[bold blue]Running general linting')
         lint_obj.lint_project(super(lint_obj.__class__, lint_obj), is_subclass_calling=False)
 
         # Run the project specific linting
+        log.debug(f'Running linting of {template_handle}')
         print(f'[bold blue]Running {template_handle} linting')
 
         lint_obj.lint()
